% Jonathan D. Hauenstein
% Copyright 2011
% Compute the Hilbert function of a set of points in projective space

function [HF, SV] = HilbertPoints(Points, tol, maxDeg)

    % find the number of Points
    numPoints = size(Points, 1);

    % allocate HF
    HF = zeros(maxDeg+1,1);

    % compute intrinsic coordinates
    V = VeroneseMatrix(Points, 1);
    N = null(V);
    if size(N,2) == 0
        % full rank
        Int = Points;
    else
        % work intrinsically
        N = null(transpose(N));
        Int = transpose(N \ transpose(Points));
    end;
    
    % compute the Hilbert function using the intrinsic coordinates
    HF(1) = 1;
    SV = struct('maxSV',1,'minNonzeroSV',1,'maxZeroSV',0);
    disp('HF[0] = 1');
    
    for degree = 1:maxDeg
        if degree > 1 && HF(degree) == numPoints
            HF(degree+1) = numPoints;
            disp(['HF[',num2str(degree),'] = ',num2str(HF(degree+1))]);
        elseif degree > 1 && HF(degree) == HF(degree-1)
            HF(degree+1) = HF(degree);
            disp(['HF[',num2str(degree),'] = ',num2str(HF(degree+1))]);
        else
%disp('setup VM');
            V = VeroneseMatrix(Int, degree);
%disp('SVD');
            S = svd(V);
     
            HF(degree+1) = sum(S > tol);
            SV.maxSV(degree+1) = S(1);
            SV.minNonzeroSV(degree+1) = S(HF(degree+1));
            if HF(degree+1) < size(S,1);
                SV.maxZeroSV(degree+1) = S(HF(degree+1)+1);
            else
                SV.maxZeroSV(degree+1) = 0;
            end;
            disp(['HF[',num2str(degree),'] = ',num2str(HF(degree+1)),'  (max: ',...
                num2str(SV.maxSV(degree+1)),' min: ',num2str(SV.minNonzeroSV(degree+1)),' zero: ',num2str(SV.maxZeroSV(degree+1)),')']);
            
            if HF(degree+1) < HF(degree)
                disp(' ');
                disp(['Error: Since the Hilbert function is nondecreasing, there is an error in degree ',num2str(degree),'!']);
                return;
            end;
        end;
    end;

return;
                   

