% Jonathan D. Hauenstein
% Copyright 2011
% Load a witness set stored in the file named witSetFile to a structure W

function [W] = LoadWitnessSet(witSetFile)

    % open the file
    IN = fopen(witSetFile, 'r');
    if IN == -1
        % file does not exist
        disp(['The file ''', witSetFile, ''' does not exist!']);
        W = 0;
        return;
    end;

    % read in the number of variables
    numVars = fscanf(IN, '%d', 1);

    % read in the number of codimensions
    numCodim = fscanf(IN, '%d', 1);

    % read in the codim
    for j = 1:numCodim
        % read in the codim & number of points
        codimNum = fscanf(IN, '%d', 1);
        dimension = numVars - codimNum - 1;
        numPoints = fscanf(IN, '%d', 1);

        % initialize the structures
        Points = zeros(numPoints, numVars);
        condNum = zeros(numPoints, 1);
        multiplicity = zeros(numPoints, 1);
        componentNum = zeros(numPoints, 1);

        % read in the witness points
        for k = 1:numPoints
            % read in the precision - ignore it
            fscanf(IN, '%d', 1); 
            % read in the coordinates
            for l = 1:numVars
                % read in the point
                P = fscanf(IN, '%le', 2);
                Points(k, l) = P(1) + 1i*P(2);
            end;

            % other approximation
            fscanf(IN, '%d', 1); 
            for l = 1:numVars
                % read in the point
                fscanf(IN, '%le', 2);
            end;

            % read in the condition number
            condNum(k) = fscanf(IN, '%le', 1);

            % ignore next 4 data
            fscanf(IN, '%le', 4);

            % read in multiplicity & component number
            multiplicity(k) = fscanf(IN, '%d', 1);
            componentNum(k) = fscanf(IN, '%d', 1);

            % ignore next data
            fscanf(IN, '%d', 1);
        end;

        % count the number of components
        numComponents = max(componentNum) + 1;

        % setup the degrees of the components
        degrees = zeros(numComponents, 1);
        for k = 1:numPoints
          if componentNum(k) >= 0
            degrees(componentNum(k)+1) = degrees(componentNum(k)+1) + 1;
          end;
        end;

if j == 2 C = componentNum; end;
        % setup the structure
        Codim{j} = struct('dimension', dimension, 'numPoints', numPoints, 'numComponents', numComponents, ...
            'degrees', degrees, 'Points', Points, 'conditionNum', condNum, 'multiplicity', multiplicity, 'componentNum', componentNum);
    end;

    W = struct('numDimensions', {numCodim}, 'witnessData', {0});
    W.witnessData = Codim;

    % display summary
    disp(' ');
    disp('**************** Summary ****************');
    disp(' ');
    for j = 1:numCodim
        if W.witnessData{j}.numComponents > 0
            if W.witnessData{j}.numComponents > 1
                S = 's';
            else
                S = '';
            end;
            disp(['Dimension ',num2str(W.witnessData{j}.dimension),': ',num2str(W.witnessData{j}.numComponents),' classified component',S]);
            disp('-----------------------------------------');

            m = min(W.witnessData{j}.degrees);
            M = max(W.witnessData{j}.degrees);

            for k = m:M
                count = sum(W.witnessData{j}.degrees == k);
                if count > 0
                    if count > 1
                        S = 's';
                    else
                        S = '';
                    end;
                    disp(['   degree ',num2str(k),': ',num2str(count),' component',S]); 
                end;
            end;        
            disp(' ');
        end;
    end;

return;

