function [VM] = VeroneseMatrix(Pts, deg, normalize)
    % compute the Veronese embedding of 'Pts' in degree 'deg'

    % default
    if nargin < 3
        normalize = 1;
    end;

    N = size(Pts,1); % number of points
    numVars = size(Pts,2); % number of variables

    % initialize VM
    VM = zeros(N,nchoosek(numVars + deg - 1, deg));
%disp(['size: ',num2str(size(VM,1)),' x ',num2str(size(VM,2))]);

%disp('setup degree');
    % setup A
    A = setupDegree(deg, numVars);

    % loop to setup VM
    for j = 1:N
%if mod(j,10000) == 1 disp([num2str(j),' of ',num2str(N)]); end;
        % normalize Pts
        if normalize == 0
            P = Pts(j,:);
        else
            P = Pts(j,:) / norm(Pts(j,:));
        end;
        
        % compute embedding
        if normalize == 0
            VM(j,:) = veronese(P,A);
        else
            v = veronese(P,A);
            VM(j,:) = v/norm(v);
        end;
    end;
return;

function [M] = veronese(X, A)
    % compute the Veronese embedding based on A

    N = size(A,1);
    M = zeros(1,N);
    for j = 1:N
        M(1,j) = prod(X.^A(j,:));
    end;
return;

function [A] = setupDegree(deg, n)
    % setup degrees for Veronese embedding

    numRows = nchoosek(n+deg-1,deg);
    A = zeros(numRows,n);
    A(1,1) = deg;

    for j = 1:numRows-1
        % copy
        A(j+1,:) = A(j,:);
        % find target entry
        target = -1;
        for k = n-1:-1:1
            if A(j+1,k) ~= 0
                target = k;
                break;
            end;
        end;
        % move entry
        A(j+1,target) = A(j+1,target) - 1;
        A(j+1,target+1) = A(j+1,target+1) + 1;
        % move all of final entry to target+1 if not the same!
        if target ~= n-1
            A(j+1,target+1) = A(j+1,target+1) + A(j+1,n);
            A(j+1,n) = 0;
        end;
    end;
return;


